@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 // 
-//  $Id: Image.cc,v 1.2 2002/01/13 18:59:39 vanrijn Exp $
+//  $Id: Image.cc,v 1.5 2002/06/07 19:07:34 eckzor Exp $
 
 // stupid macros needed to access some functions in version 2 of the GNU C
 // library
@@ -28,7 +28,7 @@
 #endif // _GNU_SOURCE
 
 #ifdef    HAVE_CONFIG_H
-#  include "../config.h"
+#  include "config.h"
 #endif // HAVE_CONFIG_H
 
 #include "BaseDisplay.hh"
@@ -1923,8 +1923,6 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	    colors[i].flags = DoRed|DoGreen|DoBlue;
 	  }
 
-      basedisplay->grab();
-
       for (i = 0; i < ncolors; i++)
 	if (! XAllocColor(basedisplay->getXDisplay(), getColormap(),
                           &colors[i])) {
@@ -1934,8 +1932,6 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	} else
 	  colors[i].flags = DoRed|DoGreen|DoBlue;
 
-      basedisplay->ungrab();
-
       XColor icolors[256];
       int incolors = (((1 << screen_depth) > 256) ? 256 : (1 << screen_depth));
 
@@ -2017,7 +2013,6 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	red_color_table[i] = green_color_table[i] = blue_color_table[i] =
 	  i / bits;
 
-      basedisplay->grab();
       for (i = 0; i < ncolors; i++) {
 	colors[i].red = (i * 0xffff) / (colors_per_channel - 1);
 	colors[i].green = (i * 0xffff) / (colors_per_channel - 1);
@@ -2033,8 +2028,6 @@ BImageControl::BImageControl(BaseDisplay *dpy, ScreenInfo *scrn, Bool _dither,
 	  colors[i].flags = DoRed|DoGreen|DoBlue;
       }
 
-      basedisplay->ungrab();
-
       XColor icolors[256];
       int incolors = (((1 << screen_depth) > 256) ? 256 :
 		      (1 << screen_depth));
@@ -2336,8 +2329,6 @@ void BImageControl::getGradientBuffers(unsigned int w,
 
 
 void BImageControl::installRootColormap(void) {
-  basedisplay->grab();
-
   Bool install = True;
   int i = 0, ncmap = 0;
   Colormap *cmaps =
@@ -2353,8 +2344,6 @@ void BImageControl::installRootColormap(void) {
 
     XFree(cmaps);
   }
-
-  basedisplay->ungrab();
 }
 
 
@@ -2381,7 +2370,7 @@ unsigned long BImageControl::getSqrt(unsigned int x) {
 }
 
 
-void BImageControl::parseTexture(BTexture *texture, char *t) {
+void BImageControl::parseTexture(BTexture *texture, const char *t) {
   if ((! texture) || (! t)) return;
 
   int t_len = strlen(t) + 1, i;
@@ -2447,7 +2436,7 @@ void BImageControl::parseTexture(BTexture *texture, char *t) {
 }
 
 
-void BImageControl::parseColor(BColor *color, char *c) {
+void BImageControl::parseColor(BColor *color, const char *c) {
   if (! color) return;
 
   if (color->isAllocated()) {
