@@ -21,7 +21,7 @@
 //
 // (See the included file COPYING / GPL-2.0)
 //
-// $Id: bbkeys.cc,v 1.14 2002/01/13 18:59:39 vanrijn Exp $
+// $Id: bbkeys.cc,v 1.36 2002/10/15 19:23:16 vanrijn Exp $
 
 #ifdef		HAVE_CONFIG_H
 #	 include "config.h"
@@ -64,14 +64,23 @@
 #	 include <sys/stat.h>
 #endif									// HAVE_SYS_STAT_H
 
+extern "C" {
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>
 #include <X11/keysym.h>
+}
 
 #include "bbkeys.hh"
+#include "stackmenu.hh"
+#include "resource.hh"
 #include "main.hh"
 #include "Timer.hh"
 #include "Basemenu.hh"
+#include "wminterface.hh"
+
+#include <string>
+
+using std::string;
 
 /*--------------------------------------------------------------------*/
 
@@ -157,7 +166,7 @@ Boston, MA 02111-1307, USA.	 */
 static int MetaMask, HyperMask, SuperMask, AltMask, ModeMask;
 static int NumLockMask, ScrollLockMask;
 
-char *ToolWindow::index_to_name(int indice)
+const char *ToolWindow::index_to_name(int indice)
 {
 	 switch (indice) {
 	 case ShiftMapIndex:
@@ -181,7 +190,7 @@ char *ToolWindow::index_to_name(int indice)
 	 }
 }
 
-void ToolWindow::x_reset_modifier_mapping(Display * display)
+void ToolWindow::x_reset_modifier_mapping(Display * d)
 {
 	 int modifier_index, modifier_key, column, mkpm;
 	 int warned_about_overlapping_modifiers = 0;
@@ -194,7 +203,7 @@ void ToolWindow::x_reset_modifier_mapping(Display * display)
 	 int mode_bit = 0;
 	 int num_lock_bit = 0;
 	 int scroll_lock_bit = 0;
-	 XModifierKeymap *x_modifier_keymap = XGetModifierMapping(display);
+	 XModifierKeymap *x_modifier_keymap = XGetModifierMapping(d);
 
 #define modwarn(name,old,other)								\
 	warned_about_overlapping_modifiers = 1
@@ -233,7 +242,7 @@ void ToolWindow::x_reset_modifier_mapping(Display * display)
 							x_modifier_keymap->modifiermap[modifier_index * mkpm +
 																	modifier_key];
 					 KeySym sym =
-							(code ? XKeycodeToKeysym(display, code, column) : 0);
+							(code ? XKeycodeToKeysym(d, code, column) : 0);
 					 if (sym == last_sym)
 							continue;
 					 last_sym = sym;
@@ -408,7 +417,7 @@ void ToolWindow::activateKeygrabs(void)
 
 	 if (grabSet.instructCount > 0)
 			XUngrabKey(getXDisplay(), AnyKey, AnyModifier,
-						 getScreenInfo(0)->getRootWindow());
+						 getCurrentScreenInfo()->getRootWindow());
 	 for (i = 0; i < grabSet.instructCount; i++) {
 			if (grabSet.KeyMap[i].keycode == 0)
 				continue;
@@ -622,79 +631,66 @@ int ToolWindow::translateAction(char *action)
 	 return 0;
 }
 
-void ToolWindow::execCommand(char *ptrCommand)
+void ToolWindow::execCommand(char *const ptrCommand)
 {
-	 int pid;
-	 extern char **environ;
+	int pid;
+	extern char **environ;
 
-	 pid = fork();
-	 if (pid == -1) {
-			fprintf(stderr,
-					 "bbkeys: Could not fork a process for execCommand.\n");
-			return;
-	 }
+	pid = fork();
+	if (pid == -1) {
+		fprintf(stderr,
+						"bbkeys: Could not fork a process for execCommand.\n");
+		return;
+	}
 
-	 if (pid == 0) {
-			char *argv[4];
-			argv[0] = "sh";
-			argv[1] = "-c";
-			argv[2] = ptrCommand;
-			argv[3] = 0;
-			execve("/bin/sh", argv, environ);
-			exit(127);
-	 }
+	if (pid == 0) {
+		const char *const argv[] = {
+      "sh",
+      "-c",
+      ptrCommand,
+      0
+    };
+		execve("/bin/sh", (char *const *)argv, environ);
+		exit(127);
+	}
 }
 
 void ToolWindow::setKeygrabs(void)
 {
-	 int res, pid;
+	int res, pid;
 
-	 pid = fork();
-	 if (pid == -1) {
-			fprintf(stderr,
-					 "bbkeys: Could not fork a process for configurator.\n");
-			return;
-	 }
+	pid = fork();
 
-	 if (pid == 0) {
-			if (noQt) {
-				// Make 'bbconf' automagically fail...
-				res = 1;
-			} else {
-				char *begin="keybindings:loadfile=";
-		    char *args = new char[strlen(begin) + strlen(bbkeys_rcfile) +1];
-				sprintf(args, "%s%s", begin, bbkeys_rcfile);
-
-				res = execlp("bbconf", "bbconf", "--start-plugin", "key bindings", 
-				"--args", args, NULL);
-
-				delete args;
-			}
+	if (pid == -1) {
+		fprintf(stderr,
+		"bbkeys: Could not fork a process for configurator.\n");
+		return;
+	}
+	
+	if (pid == 0) {
+		if (noQt) {
+			// Make 'bbconf' automagically fail...
+			res = 1;
+		} else {
+			string ost = (string)"keybindings:loadfile=" + bbkeys_rcfile;
+			
+			res = execlp("bbconf", "bbconf", "--start-plugin", "key bindings", 
+				"--args", ost.c_str(), NULL);
+			
+		}
+		
+		if (res != 0) {
+			string ost = (string)"bbkeysconf.pl -rcfile " + bbkeys_rcfile;
 
+			res = execlp("rxvt", "rxvt", "-bg", "black", "-fg", "green",
+				"-e", "sh", "-c", ost.c_str(), NULL);
 			if (res != 0) {
-				char *begin="bbkeysconf.pl -rcfile ";
-		    char *args = new char[strlen(begin) + strlen(bbkeys_rcfile) +1];
-				sprintf(args, "%s%s", begin, bbkeys_rcfile);
-			 	res = execlp("rxvt", "rxvt", "-bg", "black", "-fg", "red",
-								"-e", args, NULL);
-			 	if (res != 0) {
-						execlp("xterm", "xterm", "-bg", "black", "-fg",
-								"red", "-e", args, NULL);
-			 	}
-				delete args;
+				execlp("xterm", "xterm", "-bg", "black", "-fg", "green", 
+					"-e", "sh", "-c", ost.c_str(), NULL);
 			}
-			exit(0);
-	 }
-
-	 /*
-	 do {
-			if (waitpid(pid, &status, 0) == -1) {
-				if (errno != EINTR)
-					 return;
-			} else
-				return;
-	 } while (1);
-	 */
+		}
+		exit(0);
+	}
 
 }
 
@@ -780,7 +776,7 @@ void ToolWindow::loadKeygrabs(void)
 					 execCmdBegin = strchr(actionBegin + 1, '(');
 					 if (execCmdBegin) {
 							strncpy(execCommand, execCmdBegin + 1,
-									 strcspn(execCmdBegin + 1, ")"));
+									 strrchr(execCmdBegin, ')') - execCmdBegin - 1);
 							grabSet.KeyMap[count].execCommand =
 								strdup(execCommand);
 					 }
@@ -1131,13 +1127,13 @@ void ToolWindow::MakeWindow(bool reconfigure)
 									geom_closeBtn.height);
 	 }
 
-	 char *name = BBTOOL;
+	 char *name = (char *)BBTOOL;
 	 XSizeHints sizehints;
 
 	 wmhints.flags = StateHint;
 
-	 classhints.res_name = BBTOOL;
-	 classhints.res_class = "bbtools";
+	 classhints.res_name = (char *)BBTOOL;
+	 classhints.res_class = (char *)"bbtools";
 
 	 sizehints.x = frame.x;			//getResource()->position.x;
 	 sizehints.y = frame.y;			//getResource()->position.y;
@@ -1242,7 +1238,7 @@ void ToolWindow::Redraw()
 	XClearWindow(getXDisplay(), win_configBtn);
 	XClearWindow(getXDisplay(), win_closeBtn);
 
-	char *title = "bbkeys";
+	char *title = (char *)"bbkeys";
 	XDrawString(getXDisplay(), win_title, frameGC, geom_title.x - 1,
 			(geom_title.height + resource->label.font->ascent -
 			resource->label.font->descent) / 2, title, strlen(title));
@@ -1332,6 +1328,106 @@ unsigned int ToolWindow::KeycodeToModmask(unsigned int code)
 	return (unsigned int)-1;
 }
 
+
+void ToolWindow::findFramePosition(Window window, int &x, int &y) {
+  Window win = window, parent, root, last = None;
+  Window *children = 0;
+  unsigned int nchildren;
+  int gravity, top, bottom, left, right;
+  XWindowAttributes wattr;
+  XSizeHints size;
+  long ret;
+  unsigned int cwidth, cheight;
+  
+  // get the location, size and gravity of the client window
+  if (! XGetWindowAttributes(getXDisplay(), window, &wattr)) return;
+  cwidth = wattr.width;
+  cheight = wattr.height;
+  if (! XGetWMNormalHints(getXDisplay(), window, &size, &ret)) return;
+  if (size.flags & PWinGravity)
+    gravity = size.win_gravity;
+  else
+    gravity = NorthWestGravity;
+    
+  while (XQueryTree(getXDisplay(), win, &root, &parent, &children,
+                    &nchildren)) {
+    if (children && nchildren > 0)
+      XFree(children); // don't care about the children
+
+    if (! parent) // no parent!?
+      return;
+
+    // if the parent window is the root window, stop here
+    if (parent == root)
+      break;
+
+    last = win;
+    win = parent;
+  }
+
+  if (! (XTranslateCoordinates(getXDisplay(), last, win, 0, 0, &left, &top,
+                               &parent) &&
+         XGetWindowAttributes(getXDisplay(), win, &wattr)))
+    return;
+
+  right = wattr.width - cwidth - left;
+  bottom = wattr.height - cheight - top;
+
+  left += wattr.border_width;
+  right += wattr.border_width;
+  top += wattr.border_width;
+  bottom += wattr.border_width;
+
+  // find the client's location
+  x = wattr.x + left;
+  y = wattr.y + top;
+
+  // this makes things work. why? i don't know. but you need them.
+  right -= 2;
+  bottom -= 2;
+
+  // find the frame's reference position based on the window's gravity
+  switch (gravity) {
+  case NorthWestGravity:
+    x -= left;
+    y -= top;
+    break;
+  case NorthGravity:
+    x += (left + right) / 2;
+    y -= top;
+    break;
+  case NorthEastGravity:
+    x += right;
+    y -= top;
+  case WestGravity:
+    x -= left;
+    y += (top + bottom) / 2;
+    break;
+  case CenterGravity:
+    x += (left + right) / 2;
+    y += (top + bottom) / 2;
+    break;
+  case EastGravity:
+    x += right;
+    y += (top + bottom) / 2;
+  case SouthWestGravity:
+    x -= left;
+    y += bottom;
+    break;
+  case SouthGravity:
+    x += (left + right) / 2;
+    y += bottom;
+    break;
+  case SouthEastGravity:
+    x += right;
+    y += bottom;
+    break;
+  default:
+    break;
+  }
+}
+
+
 void ToolWindow::process_event(XEvent * e)
 {
 	switch (e->type) {
@@ -1381,25 +1477,11 @@ void ToolWindow::process_event(XEvent * e)
 
 		int i = 0;
 		int grabInt = -1;
-		Window fw_root, fw_child;
 		int fw_y, fw_x;
-		unsigned int fw_w, fw_h, fw_b, fw_d;
 
-		/* Need to take out this next bit when nyz has 0.60.0 ready, as
-			 we'll then get our focus_window in a ClientMessage */
-		int revert_to = RevertToPointerRoot;
-		if (!focus_window) {
-			XGetInputFocus(getXDisplay(), &focus_window, &revert_to);
-		}
-
-		/* end temporary focus_window fix */
-
-		if (focus_window && focus_window != (int)PointerRoot ) {
-			XGetGeometry(getXDisplay(), focus_window, &fw_root, &fw_x,
-					&fw_y, &fw_w, &fw_h, &fw_b, &fw_d);
-			XTranslateCoordinates(getXDisplay(), focus_window, fw_root,
-					fw_x, fw_y, &fw_x, &fw_y, &fw_child);
-		}
+		if (focus_window) {
+      findFramePosition(focus_window, fw_x, fw_y);
+    }
 		
 		// if our user wants to grab his keystrokes, even though one
 		// or more of the Lock-Modifiers are pressed, alter the mask
@@ -1438,21 +1520,18 @@ void ToolWindow::process_event(XEvent * e)
 
 			switch (grabSet.KeyMap[grabInt].action) {
 			case grabIconify:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					XIconifyWindow(getXDisplay(), focus_window, 0);
-				}
 				break;
 
 			case grabRaise:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					XRaiseWindow(getXDisplay(), focus_window);
-				}
 				break;
 
 			case grabLower:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					XLowerWindow(getXDisplay(), focus_window);
-				}
 				break;
 
 			case grabClose:
@@ -1647,78 +1726,76 @@ void ToolWindow::process_event(XEvent * e)
 				break;
 
 			case grabShade:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					wminterface->shadeWindow(focus_window);
-				}
 				break;
 
 			case grabStick:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					wminterface->stickWindow(focus_window);
-				}
 				break;
 
 			case grabExecute:
-				execCommand(grabSet.KeyMap[grabInt].execCommand);
+				execCommand((char *const)grabSet.KeyMap[grabInt].execCommand);
 				break;
 
 			case grabMaximize:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					wminterface->maximizeWindow(focus_window, True, True);
 				break;
 
 			case grabVertMax:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					wminterface->maximizeWindow(focus_window, False, True);
 				break;
 
 			case grabHorizMax:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					wminterface->maximizeWindow(focus_window, True, False);
 				break;
 		
 			case grabNudgeRight:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x + 1, fw_y);
 				break;
 
 			case grabNudgeLeft:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x - 1, fw_y);
 				break;
 
 			case grabNudgeUp:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x, fw_y - 1);
 				break;
 
 			case grabNudgeDown:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x, fw_y + 1);
 				break;
 
 			case grabBigNudgeRight:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x + 10, fw_y);
 				break;
 
 			case grabBigNudgeLeft:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x - 10, fw_y);
 				break;
 
 			case grabBigNudgeUp:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x, fw_y - 10);
 				break;
 
 			case grabBigNudgeDown:
-				if (focus_window && focus_window != (int)PointerRoot )
+				if (focus_window)
 					XMoveWindow(getXDisplay(), focus_window, fw_x, fw_y + 10);
 				break;
 
 			case grabHorizInc:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window) {
 					XWindowAttributes foo;
 					XGetWindowAttributes(getXDisplay(), focus_window, &foo);
 					XResizeWindow(getXDisplay(), focus_window,
@@ -1727,7 +1804,7 @@ void ToolWindow::process_event(XEvent * e)
 				break;
 
 			case grabVertInc:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window) {
 					XWindowAttributes foo;
 					XGetWindowAttributes(getXDisplay(), focus_window, &foo);
 					XResizeWindow(getXDisplay(), focus_window,
@@ -1736,7 +1813,7 @@ void ToolWindow::process_event(XEvent * e)
 				break;
 
 			case grabHorizDec:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window) {
 					XWindowAttributes foo;
 					XGetWindowAttributes(getXDisplay(), focus_window, &foo);
 					if (foo.width < 11)
@@ -1747,7 +1824,7 @@ void ToolWindow::process_event(XEvent * e)
 				break;
 
 			case grabVertDec:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window) {
 					XWindowAttributes foo;
 					XGetWindowAttributes(getXDisplay(), focus_window, &foo);
 					if (foo.height < 11)
@@ -1758,9 +1835,8 @@ void ToolWindow::process_event(XEvent * e)
 				break;
 
 			case grabToggleDecor:
-				if (focus_window && focus_window != (int)PointerRoot ) {
+				if (focus_window)
 					wminterface->decorateToggleWindow(focus_window);
-				}
 				break;
 			}
 		}
@@ -1907,11 +1983,11 @@ void ToolWindow::timeout(void)
 
 /*--------------------------------*/
 
-void ToolWindow::raiseWindow(Window win)
+void ToolWindow::raiseWindow(Window)
 {
 }
 
-void ToolWindow::lowerWindow(Window win)
+void ToolWindow::lowerWindow(Window)
 {
 }
 
@@ -2031,15 +2107,20 @@ void ToolWindow::setDesktopCount(int count)
 
 /*--------------------------------------*/
 
+void ToolWindow::clearWindows(void)
+{
+	focusWindow(0);
+	while (! windowList->empty())
+		windowList->remove(0);
+}
+
 void ToolWindow::removeWindow(Window win)
 {
 	if (focus_window == win)
 		focusWindow(0);
-	LinkedListIterator<WindowList> it(windowList);
-	for (; it.current(); it++)
-		if (it.current()->win == win) {
-			windowList->remove(it.current());
-		}
+	for (int i = 0; i < windowList->count(); ++i)
+		if (windowList->find(i)->win == win)
+			windowList->remove(i--);
 }
 
 void ToolWindow::focusWindow(Window win)
@@ -2274,12 +2355,12 @@ void ToolWindow::focus_stack(Window win)
 	}
 }
 
-void ToolWindow::saveMenuSearch(Window window, Basemenu *menu)
+void ToolWindow::saveMenuSearch(Window window, Basemenu *)
 {
 	menuWin = window;
 }
 
-void ToolWindow::removeMenuSearch(Window window)
+void ToolWindow::removeMenuSearch(Window)
 {
 	menuWin = (Window)NULL;
 }
