@@ -20,10 +20,10 @@
 //
 // (See the included file COPYING / GPL-2.0)
 //
-// $Id: resource.cc,v 1.4 2002/01/13 18:59:39 vanrijn Exp $
+// $Id: resource.cc,v 1.5 2002/05/30 20:47:45 eckzor Exp $
 
 #ifdef    HAVE_CONFIG_H
-#  include "../config.h"
+#  include "config.h"
 #endif													// HAVE_CONFIG_H
 
 #ifdef   STDC_HEADERS
@@ -40,7 +40,7 @@
 
 #include "resource.hh"
 #include "blackboxstyle.hh"
-
+#include "bbkeys.hh"
 
 Resource::Resource(ToolWindow * toolwindow):BaseResource(toolwindow)
 {
